package prompt.roles.data;

import java.io.Serializable;

/**
 * This class represents a dependency between two activities.
 * 
 * @author Andrea Burattin
 * @author Marco Veluscek
 */
public class Dependency implements Serializable {

	private static final long serialVersionUID = -7098596235310156801L;

	/** A very small value that can be used for the comparison in case of very
	 * low frequencies of originators */
	private static final double epsilon = 0.005;
	
	private String sourceActivity;
	private String targetActivity;
	private MultiSet<SerializablePair<String, String>> originators = new MultiSet<SerializablePair<String,String>>();
	
	/**
	 * Class constructor of two activities
	 * 
	 * @param sourceActivity the source activity
	 * @param targetActivity the target activity
	 */
	public Dependency(String sourceActivity, String targetActivity) {
		if (!sourceActivity.contains("CLASSIFIED_[") && !sourceActivity.contains("]_CLASSIFIED")) {
			if (sourceActivity.contains("+")) {
				sourceActivity = sourceActivity.substring(0, sourceActivity.indexOf("+"));
			}
		}
		if (!targetActivity.contains("CLASSIFIED_[") && !targetActivity.contains("]_CLASSIFIED")) {
			if (targetActivity.contains("+")) {
				targetActivity = targetActivity.substring(0, targetActivity.indexOf("+"));
			}
		}
		this.sourceActivity = sourceActivity;
		this.targetActivity = targetActivity;
	}
	
	/**
	 * This method adds resources to the current dependency
	 * 
	 * @param sourceResource name of the originator performing the source
	 * activity
	 * @param targetResource name of the originator performing the target
	 * activity
	 */
	public void addResources(String sourceResource, String targetResource) {
		originators.put(new SerializablePair<String, String>(sourceResource, targetResource));
	}
	
	/**
	 * This method returns the name of the source activity of the current
	 * dependency
	 * 
	 * @return the activity name
	 */
	public String getSourceActivity() {
		return sourceActivity;
	}
	
	/**
	 * This method returns the name of the target activity of the current
	 * dependency
	 * 
	 * @return the activity name
	 */
	public String getTargetActivity() {
		return targetActivity;
	}
	
	/**
	 * This method returns the degree of <strong>no</strong> handover of role
	 * associated to the current dependency.
	 * 
	 * This method implements the <code>w_{ab}(L)</code> formula, which is the
	 * formula no. (1), reported into the related paper ("Business Models
	 * Enhancement through Discovery of Roles", CIDM 2013).
	 * 
	 * @return the degree of no handover of role
	 */
	public double getDegreeOfNoHandover() {
		MultiSet<String> U_a = getSourceOriginators();
		MultiSet<String> U_b = getTargetOriginators();
		MultiSet<String> U_ab = getUab();
		
		double num = MultiSet.intersection(U_a, U_b).size() + U_ab.size();
		double den = U_a.size() + U_b.size();
		
		double weight = num / den;
		
		if (weight <= epsilon || Double.isNaN(weight)) {
			weight = 0.0;
		}
		
		return weight;
	}
	
	/**
	 * This method returns the multiset of originators associated to the source
	 * activity
	 * 
	 * @return the multiset of originators
	 */
	public MultiSet<String> getSourceOriginators() {
		MultiSet<String> U_a = new MultiSet<String>();
		for(SerializablePair<String, String> o : originators.keySet()) {
			U_a.add(o.getFirst(), originators.get(o));
		}
		return U_a;
	}
	
	/**
	 * This method returns the multiset of originators associated to the source
	 * activity
	 * 
	 * @return the multiset of originators
	 */
	public MultiSet<String> getTargetOriginators() {
		MultiSet<String> U_b = new MultiSet<String>();
		for(SerializablePair<String, String> o : originators.keySet()) {
			U_b.add(o.getSecond(), originators.get(o));
		}
		return U_b;
	}
	
	/**
	 * This method returns the <code>U_{ab}(L)</code> measure described in the
	 * paper
	 * 
	 * @return the <code>U_{ab}(L)</code> value
	 */
	private MultiSet<String> getUab() {
		MultiSet<String> U_ab = new MultiSet<String>();
		for(SerializablePair<String, String> o : originators.keySet()) {
			if (o.getFirst() != null && o.getSecond() != null && o.getFirst().equals(o.getSecond())) {
				U_ab.add(o.getSecond(), originators.get(o));
			}
		}
		return U_ab;
	}
	
	@Override
	public boolean equals(Object other) {
		if (other instanceof Dependency) {
			if (((Dependency) other).sourceActivity.equals(sourceActivity) &&
				((Dependency) other).targetActivity.equals(targetActivity)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public int hashCode() {
		return (sourceActivity + targetActivity).hashCode();
	}
	
	@Override
	public String toString() {
		return sourceActivity + " -> " + targetActivity + " (w_ab = " + getDegreeOfNoHandover() + ", " + originators + ")";
	}
}
