package prompt.roles.data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * This class represents a general multiset. A multiset, in this context, is
 * basically a set in which, each element, has a multiplicity.
 * 
 * @author Marco Veluscek
 * @author Andrea Burattin
 * @param <T> the type of elements to store into the multiset
 */
public class MultiSet<T extends Serializable> extends HashMap<T, Integer> {

	private static final long serialVersionUID = -7593981219834036213L;

	/**
	 * This method adds and, in case, increments the frequency of an element
	 * into the current multiset
	 * 
	 * @param element the element to add
	 */
	public void put(T element) {
		add(element, 1);
	}
	
	/**
	 * This method adds and, in case, increments the frequency of an element
	 * into the current multiset by the provided increment factor
	 * 
	 * @param element the element to add
	 * @param incrementFactor the factor to sum
	 * @return the previous value associated with <tt>element</tt>
	 */
	public Integer add(T element, Integer incrementFactor) {
		if (containsKey(element)) {
			Integer oldValue = get(element);
			super.put(element, oldValue + incrementFactor);
			return oldValue;
		} else {
			super.put(element, incrementFactor);
		}
		return 0;
	}
	
	/**
	 * This static method merges the two given multisets.
	 * 
	 * @param set1 the first set
	 * @param set2 the second set
	 * @return the union of the two multisets
	 */
	public static <T extends Serializable> MultiSet<T> union(MultiSet<T> set1, MultiSet<T> set2) {
		MultiSet<T> toReturn = new MultiSet<T>();
		
		for (T key : set1.keySet()) {
			toReturn.add(key, set1.get(key));
		}
		for (T key : set2.keySet()) {
			toReturn.add(key, set2.get(key));
		}
		return toReturn;
	}
	
	/**
	 * This static method intersects two multisets
	 * 
	 * @param set1 the first set
	 * @param set2 the second set
	 * @return the intersection of the two multiset
	 */
	public static <T extends Serializable> MultiSet<T> intersection(MultiSet<T> set1, MultiSet<T> set2) {
		MultiSet<T> intersection = new MultiSet<T>();
		
		// Go through set 1
		for (Map.Entry<T, Integer> elem : set1.entrySet()) {
			T key = elem.getKey();
			// If set1.key is in set2..
			if (set2.containsKey(key)) {
				// Take the min occurrence
				if (elem.getValue() <= set2.get(key)) {
					intersection.put(key, elem.getValue());
				} else {
					intersection.put(key, set2.get(key));
				}
			}
		}
		return intersection;
	}
	
	/*/**
	 * This static method intersects the simple sets of two multisets and
	 * returns a multiset with multiplicity set to 1 to all values
	 * 
	 * @param set1 the first set
	 * @param set2 the second set
	 * @return the intersection of the two multiset, where each element has
	 * multiplicity 1
	 */
	/*public static <T> MultiSet<T> intersectSimpleSet(MultiSet<T> set1, MultiSet<T> set2) {
		MultiSet<T> intersection = new MultiSet<T>();
		// Go through set 1
		for (T key : set1.keySet()) {
			// If set1.key is in set2..
			if (set2.containsKey(key)) {
				intersection.put(key, 1);
			}
		}
		return intersection;
	}*/
	
	/**
	 * This method removes the given element the number of times provided
	 * 
	 * @param key the element to remove
	 * @param freq the number of occurrences to remove
	 */
	public void remove(T key, int freq) {
		if (containsKey(key)) {
			int actFreq = get(key);
			if (actFreq - freq <= 0) {
				remove(key);
			} else {
				put(key, actFreq - freq);
			}
		}
	}
	
	/**
	 * This method returns the set of elements without multiplicity
	 * 
	 * @return the set without any multiplicity
	 */
	public HashSet<T> getSimpleSet() {
		return new HashSet<T>(keySet());
	}
	
	@Override
	public int size() {
		int size = 0;
		for (Integer occ : values()) {
			size += occ;
		}
		return size;
	}
	
	@Override
	public boolean equals(Object other) {
		if (other instanceof MultiSet<?>) {
			MultiSet<?> set2 = (MultiSet<?>)other;
			if (size() != set2.size()) {
				return false;
			}
			for (T elem : keySet())  {
				if (!get(elem).equals(set2.get(elem))) {
					return false;
				}
			}
			return true;
		}
		return false;
	}
	
	@Override
	public String toString() {
		String set = "";
		for (T el : keySet()) {
			set += "(" + el.toString() + ": " + get(el).toString() + "), ";
		}
		if (set.length() > 3) {
			set = set.substring(0, set.length() - 2);
		}
		return set;
	}
}
