package prompt.roles.data;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * This class describes a general role. A role, in this context, is a set of
 * strings. Each string represents the name of one activity.
 * 
 * @author Andrea Burattin
 * @author Marco Veluscek
 */
public class Role extends HashSet<String> {

	private static final long serialVersionUID = -1240986528520010753L;
	private MultiSet<String> originators = new MultiSet<String>();
	private Set<Dependency> dependencies = new HashSet<Dependency>();
	
	/**
	 * This method adds the dependency to the set of dependencies of this role
	 * 
	 * @param dependency the dependecy to add
	 */
	public void addDependency(Dependency dependency) {
		dependencies.add(dependency);
	}
	
	/**
	 * This method adds the provided activity to the current role
	 * 
	 * @param activity the activity to add
	 */
	public void addActivity(String activity) {
		add(activity);
	}
	
	/**
	 * This method returns the {@link MultiSet} of originators of the current
	 * role
	 * 
	 * @return the multiset of originators
	 */
	public MultiSet<String> getOriginators() {
		return this.originators;
	}
	
	/**
	 * This method computes the weight of the current role
	 * 
	 * @return the weight of the role
	 */
	public double getWeight() {
		double weight = 0.0;
		for (Dependency dependency : dependencies) {
			weight += (1 - dependency.getDegreeOfNoHandover());
		}
		return weight;
	}
	
	/**
	 * Method to calculate the amount of shared originators between the two
	 * roles.
	 * 
	 * This method implements the <code>rho_{r1, r2}(L)</code> formula, which is
	 * the formula no. (2), reported into the related paper ("Business Models
	 * Enhancement through Discovery of Roles", CIDM 2013).
	 * 
	 * @param role1 the first role
	 * @param role2 the second role
	 * @return the amount of shared originators
	 */
	public static double rho(Role role1, Role role2) {
		double intersecSize = MultiSet.intersection(role1.originators, role2.originators).size();
		double sum = role1.originators.size() + role2.originators.size();
		
		return (2.0 * intersecSize) / (sum);
	}
	
	/**
	 * This method merges two roles, and creates a new one.
	 * 
	 * @param role1 the first role
	 * @param role2 the second role
	 * @return the role obtained by merging the two provided roles
	 */
	public static Role merge(Role role1, Role role2) {
		Role role = new Role();
		role.addAll(role1);
		role.addAll(role2);
		role.dependencies.addAll(role1.dependencies);
		role.dependencies.addAll(role2.dependencies);
		role.originators = MultiSet.union(role1.originators, role2.originators);
		return role;
	}
	
	/**
	 * This method provides a string representation of two roles.
	 * 
	 * @param r1 the first role
	 * @param r2 the second role
	 * @return a string representation of two roles
	 */
	public static String stringRepresentation(Role r1, Role r2) {
		List<String> list = new LinkedList<String>();
		list.addAll(r1);
		list.addAll(r2);
		Collections.sort(list);
		
		StringBuilder sb = new StringBuilder();
		for(String s : list) {
			sb.append(s);
		}
		return sb.toString();
	}
	
	@Override
	public String toString() {
		return super.toString() + " {{ " + originators.toString() + " }}";
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}
	
	@Override
	public boolean equals(Object other) {
		if (other instanceof Role) {
			if (size() != ((Role) other).size()) {
				return false;
			}
			for (String s : this) {
				if (!((Role) other).contains(s)) {
					return false;
				}
			}
			return true;
		}
		return false;
	}
}
