package prompt.roles.plugins;

import java.text.ParseException;

import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryBufferedImpl;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.guards.Expression;

import prompt.roles.data.Role;
import prompt.roles.data.RoleSet;
import prompt.roles.utils.AuthorsInfo;

/**
 * This class contains the plugin that is used to decorate a Data Petri Net,
 * with the provided set of roles
 * 
 * @author Andrea Burattin
 */
public class DecorateDataPetriNet {

	private static XFactory factory = new XFactoryBufferedImpl();
	
	/**
	 * This method is the actual plugin
	 * 
	 * @param context
	 * @param model
	 * @param roles
	 * @return
	 * @throws ParseException 
	 */
	@Plugin(
		name = "Decorate a Data Petri Net with a Role Partition",
		parameterLabels = { "A Data Petri Net", "A roles partition" },
		returnLabels = { "The decorated Data Petri Net" },
		returnTypes = { PetriNetWithData.class },
		userAccessible = true
	)

	@UITopiaVariant(
		affiliation = AuthorsInfo.AFFILIATION,
		author = AuthorsInfo.AUTHOR, 
		email = AuthorsInfo.EMAIL,  
		pack = AuthorsInfo.PACK, 
		website = AuthorsInfo.WEBSITE
	)
	public static PetriNetWithData decorateModel(UIPluginContext context, PetriNetWithData model, RoleSet roles) throws ParseException {
		// adding guards
		DataElement dataElem=model.addVariable("Originator", String.class, null, null);
		for (Transition t : model.getTransitions()) {
			model.assignWriteOperation(t,dataElem);
			Expression expr = ((PNWDTransition)t).getGuard();
			if (!t.isInvisible()) {
				String activityName = t.getLabel();
				Role r = roles.search(activityName);
				if (r != null) {
					for(String originator : r.getOriginators().keySet()) {
						if (expr == null) {
							expr = new Expression("(Originator' == \"" + originator + "\")");
						} else {
							Expression newTerm = new Expression("(Originator' == \"" + originator + "\")");
							expr = new Expression(expr, "||", newTerm);
						}
					}
				}
			}
			if (expr != null) {
				model.setGuard(t, expr.toString());
			}
		}
		return model;
	}
	
	@Plugin(
		name = "Preprocess log",
		parameterLabels = { "The original log" },
		returnLabels = { "The decorated log" },
		returnTypes = { XLog.class },
		userAccessible = true
	)

	@UITopiaVariant(
		affiliation = AuthorsInfo.AFFILIATION,
		author = AuthorsInfo.AUTHOR, 
		email = AuthorsInfo.EMAIL,  
		pack = AuthorsInfo.PACK, 
		website = AuthorsInfo.WEBSITE
	)
	public static XLog preprocessLog(PluginContext context, XLog log) {
		XLog clonedLog = factory.createLog();
		for(XTrace trace : log) {
			XTrace clonedTrace = factory.createTrace();
			clonedTrace.setAttributes(trace.getAttributes());
			for (XEvent event : trace) {
				XEvent clonedEvent = factory.createEvent();
				clonedEvent.setAttributes(event.getAttributes());
				clonedEvent.getAttributes().put("Originator", new XAttributeLiteralImpl("Originator", XOrganizationalExtension.instance().extractResource(event)));
				clonedTrace.add(clonedEvent);
			}
			clonedLog.add(clonedTrace);
		}
		return clonedLog;
	}
}
