package prompt.roles.utils;

import java.util.Map;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.elements.Activity;
import org.processmining.models.graphbased.directed.bpmn.elements.Flow;

import prompt.roles.data.Dependency;
import prompt.roles.data.DependencySet;
import prompt.roles.data.MultiSet;
import prompt.roles.plugins.PreprocessModel;

/**
 * This class is responsible of the extraction of the dependencies starting from
 * a {@link BPMNDiagram}.
 * 
 * @author Andrea Burattin
 * @author Marco Veluscek
 */
public class DependencyExtractor {

	private BPMNDiagram model;
	private DependencySet dependencies = null;
	
	/**
	 * Main class constructor
	 * 
	 * @param model the model to extract the dependencies from
	 */
	public DependencyExtractor(BPMNDiagram model) {
		this.model = model;
	}
	
	/**
	 * This method returns the set of dependencies associated to the provided
	 * model
	 * 
	 * @return the set of dependencies of the BPMN model.
	 */
	public DependencySet getDependencies() {
		if (dependencies == null) {
			dependencies = searchDependencies();
		}
		return dependencies;
	}
	
	/**
	 * This method weights each dependency already discovered according to the
	 * data contained into the provided log
	 * 
	 * @param log the log to use for the dependency weighting
	 * @param activityOriginatorFrequencies this object is modified in order to
	 * count the number of events that each originator performed for each
	 * activity
	 * @return the new set of weighted dependencies
	 */
	public DependencySet getWeightedDependencies(XLog log, Map<String, MultiSet<String>> activityOriginatorFrequencies) {
		DependencySet weightedDependencies = new DependencySet();
		
		for (Dependency d : getDependencies()) {
			weightedDependencies.add(new Dependency(d.getSourceActivity(), d.getTargetActivity()));
		}
		
		for(XTrace t : log) {
			XEvent secondToLastEvent = null;
			for (XEvent lastEvent : t) {
				String target = XLogHelper.getName(lastEvent);
				String resourceTarget = XLogHelper.getResource(lastEvent);
				
				if (!activityOriginatorFrequencies.containsKey(target)) {
					activityOriginatorFrequencies.put(target, new MultiSet<String>());
				}
				activityOriginatorFrequencies.get(target).put(resourceTarget);
				
				if (secondToLastEvent != null) {
					String source = XLogHelper.getName(secondToLastEvent);
					Dependency currentDependency = weightedDependencies.search(source, target);
					
					if (currentDependency != null) {
						String resourceSource = XLogHelper.getResource(secondToLastEvent);
						if (resourceSource != null && resourceTarget != null) {
							currentDependency.addResources(resourceSource, resourceTarget);
							weightedDependencies.add(currentDependency);
						}
					}
				}
				secondToLastEvent = lastEvent;
			}
		}
		
		return weightedDependencies;
	}
	
	/**
	 * This method is actually responsible of extracting the dependencies
	 * 
	 * @return the set of dependencies
	 */
	private DependencySet searchDependencies() {
		DependencySet dependencies = new DependencySet();
		
		PreprocessModel preprocessor = new PreprocessModel();
		BPMNDiagram preprocessesDiagram = preprocessor.preprocess(model);
		
		for(Flow f : preprocessesDiagram.getFlows()) {
			if (f.getSource() instanceof Activity && f.getTarget() instanceof Activity) {
				Activity source = (Activity) f.getSource();
				Activity target = (Activity) f.getTarget();
				if (!PreprocessModel.avoidNode(source.getLabel()) && !PreprocessModel.avoidNode(target.getLabel())) {
					dependencies.add(new Dependency(source.getLabel(), target.getLabel()));
				}
			}
		}
		
		return dependencies;
	}
}
