package prompt.roles.utils;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

import prompt.roles.data.MultiSet;
import prompt.roles.data.Role;
import prompt.roles.data.RoleSet;

/**
 * This class manages the standard entropy concepts
 * 
 * @author Marco Veluscek
 * @author Andrea Burattin
 */
public class Entropy {

	private static double gamma = 0.5;
	
	/**
	 * This method computes the entropy as described in the formula (3) of the
	 * related paper ("Business Models Enhancement through Discovery of Roles",
	 * CIDM 2013).
	 * 
	 * @param roles the set of roles identified
	 * @param log the log
	 * @param considerUserFrequencies whether to consider also the single
	 * frequencies of each user
	 * @return the entropy value
	 */
	public static double computeEntropy(RoleSet roles, XLog log, boolean considerUserFrequencies) {
		if (roles.size() == 1) {
			return Double.POSITIVE_INFINITY;
		}
		MultiSet<String> originators = getOriginators(log);
		double entropy = 0.0;
		for (Role role : roles) {
			double roleEntropy = 0.0;
			for (String originator : originators.keySet()) {
				if (role.getOriginators().containsKey(originator)) {
					double L_Ru = role.getOriginators().get(originator);
					double L_u = originators.get(originator);
					double tmpEntropy = computeEntropy(L_Ru, L_u);
					
					if (considerUserFrequencies) {
						double userFreq = ((double)originators.get(originator)) / ((double)originators.keySet().size());
						tmpEntropy *= userFreq;
					}
					
					roleEntropy += tmpEntropy;
				}
			}
			entropy += roleEntropy * normalizeByRole(roles, role);
		}
		
		return entropy;
	}
	
	/**
	 * This method returns the normalization factor of the provided role into
	 * a set o roles
	 * 
	 * @param partition the partition of the role
	 * @param role the role to normalize
	 * @return the normalization factor
	 */
	private static double normalizeByRole(RoleSet partition, Role role) {
		double roleWeights = role.getWeight();
		double weights = partition.getWeights();
		
		if (weights == 0) {
			return 1.0;
		}
		return 1.0 + gamma * (roleWeights / weights);
	}
	
	/**
	 * This method computes the frequencies of each originators
	 * 
	 * @param log the log to use
	 * @return a multiset with the frequency of each originator
	 */
	private static MultiSet<String> getOriginators(XLog log) {
		MultiSet<String> originators = new MultiSet<String>();
		for (XTrace t : log) {
			for (XEvent e : t) {
				originators.put(XLogHelper.getResource(e));
			}
		}
		return originators;
	}
	
	/**
	 * This method calculates the entropy of <code>num / den</code>
	 * 
	 * @param num the numerator
	 * @param den the denumerator
	 * @return the entropy value
	 */
	private static double computeEntropy(double num, double den) {
		double x = num / den;
		
		if (x == 0.0) {
			return x;
		}
		
		double first = -(x);
		double second = Math.log10(x);
		
		return first * second;
	}
}
